@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2022 Red Hat, Inc.
+// Copyright (C) 2013-2023 Red Hat, Inc.
 
 /// @file
 
@@ -11,12 +11,14 @@
 #include <stdint.h>
 #include <cstddef>
 #include <cstdlib>
+#include <regex.h>
 #include <list>
 #include <memory>
 #include <ostream>
 #include <string>
 #include <typeinfo>
 #include <unordered_map>
+#include <unordered_set>
 #include <utility> // for std::rel_ops, at least.
 #include <vector>
 #include "abg-interned-str.h"
@@ -54,10 +56,38 @@ using std::weak_ptr;
 using std::unordered_map;
 using std::string;
 using std::vector;
+using std::unordered_set;
+
+typedef unordered_set<string> string_set_type;
 
 // Pull in relational operators.
 using namespace std::rel_ops;
 
+namespace comparison
+{
+class diff_context;
+
+/// Convenience typedef for a shared pointer of @ref diff_context.
+typedef shared_ptr<diff_context> diff_context_sptr;
+
+/// Convenience typedef for a weak pointer of @ref diff_context.
+typedef weak_ptr<diff_context> diff_context_wptr;
+
+class diff;
+
+/// Convenience typedef for a shared_ptr for the @ref diff class
+typedef shared_ptr<diff> diff_sptr;
+
+/// Convenience typedef for a weak_ptr for the @ref diff class
+typedef weak_ptr<diff> diff_wptr;
+}
+
+namespace regex
+{
+/// A convenience typedef for a shared pointer of regex_t.
+typedef std::shared_ptr<regex_t> regex_t_sptr;
+}// end namespace regex
+
 namespace ir
 {
 
@@ -162,6 +192,9 @@ typedef shared_ptr<class_decl> class_decl_sptr;
 /// Convenience typedef for a vector of @ref class_decl_sptr
 typedef vector<class_decl_sptr> classes_type;
 
+/// Convenience typedef for a vector of @ref class_or_union_sptr
+typedef vector<class_or_union_sptr> classes_or_unions_type;
+
 /// Convenience typedef for a weak pointer on a @ref class_decl.
 typedef weak_ptr<class_decl> class_decl_wptr;
 
@@ -493,6 +526,9 @@ is_reference_type(const type_or_decl_base_sptr&);
 const type_base*
 is_void_pointer_type(const type_base*);
 
+const type_base*
+is_void_pointer_type(const type_base&);
+
 qualified_type_def*
 is_qualified_type(const type_or_decl_base*);
 
@@ -648,6 +684,12 @@ is_data_member(const decl_base *);
 const var_decl_sptr
 get_next_data_member(const class_or_union_sptr&, const var_decl_sptr&);
 
+var_decl_sptr
+get_last_data_member(const class_or_union&);
+
+var_decl_sptr
+get_last_data_member(const class_or_union*);
+
 var_decl_sptr
 get_last_data_member(const class_or_union_sptr&);
 
@@ -688,6 +730,13 @@ anonymous_data_member_to_class_or_union(const var_decl*);
 class_or_union_sptr
 anonymous_data_member_to_class_or_union(const var_decl_sptr&);
 
+class_or_union_sptr
+anonymous_data_member_to_class_or_union(const var_decl&);
+
+bool
+anonymous_data_member_exists_in_class(const var_decl& anon_dm,
+				      const class_or_union& clazz);
+
 bool
 scope_anonymous_or_typedef_named(const decl_base&);
 
@@ -733,6 +782,11 @@ get_data_member_offset(const decl_base_sptr);
 uint64_t
 get_absolute_data_member_offset(const var_decl&);
 
+bool
+get_next_data_member_offset(const class_or_union*,
+			    const var_decl_sptr&,
+			    uint64_t&);
+
 bool
 get_next_data_member_offset(const class_or_union_sptr&,
 			    const var_decl_sptr&,
@@ -960,6 +1014,9 @@ get_function_type_name(const function_type*, bool internal = false);
 interned_string
 get_function_type_name(const function_type&, bool internal = false);
 
+interned_string
+get_function_id_or_pretty_representation(function_decl *fn);
+
 interned_string
 get_method_type_name(const method_type_sptr&, bool internal = false);
 
@@ -1056,6 +1113,9 @@ debug(const decl_base* artifact);
 bool
 debug_equals(const type_or_decl_base *l, const type_or_decl_base *r);
 
+void
+debug_comp_stack(const environment& env);
+
 bool
 odr_is_relevant(const type_or_decl_base&);
 
@@ -1137,6 +1197,9 @@ lookup_class_type(const interned_string&, const corpus&);
 const type_base_wptrs_type*
 lookup_class_types(const interned_string&, const corpus&);
 
+const type_base_wptrs_type*
+lookup_union_types(const interned_string&, const corpus&);
+
 bool
 lookup_decl_only_class_types(const interned_string&,
 			     const corpus&,
@@ -1145,6 +1208,9 @@ lookup_decl_only_class_types(const interned_string&,
 const type_base_wptrs_type*
 lookup_class_types(const string&, const corpus&);
 
+const type_base_wptrs_type*
+lookup_union_types(const string&, const corpus&);
+
 class_decl_sptr
 lookup_class_type_per_location(const interned_string&, const corpus&);
 
@@ -1384,7 +1450,7 @@ string
 demangle_cplus_mangled_name(const string&);
 
 type_base_sptr
-type_or_void(const type_base_sptr, const environment*);
+type_or_void(const type_base_sptr, const environment&);
 
 type_base_sptr
 canonicalize(type_base_sptr);
@@ -1440,6 +1506,18 @@ types_have_similar_structure(const type_base* first,
 			     const type_base* second,
 			     bool indirect_type = false);
 
+string
+build_internal_underlying_enum_type_name(const string &base_name,
+					 bool is_anonymous,
+					 uint64_t size);
+
+var_decl_sptr
+find_first_data_member_matching_regexp(const class_or_union& t,
+				       const regex::regex_t_sptr& r);
+
+var_decl_sptr
+find_last_data_member_matching_regexp(const class_or_union& t,
+				      const regex::regex_t_sptr& regex);
 } // end namespace ir
 
 using namespace abigail::ir;
